@@ -241,8 +241,7 @@ called for this message.")
 		  (delete-region (+ (match-beginning 0) 2)
 				 (+ (match-beginning 0) 3))))))
 	    (setq charset (mm-encode-body))
-	    (setq encoding (mm-body-encoding charset 
-					     (cdr (assq 'encoding cont))))
+	    (setq encoding (mm-body-encoding charset))
 	    (setq coded (buffer-string)))
 	(mm-with-unibyte-buffer
 	  (cond
@@ -301,6 +300,7 @@ called for this message.")
         (let ((mml-boundary (mml-compute-boundary cont)))
           (insert (format "Content-Type: multipart/%s; boundary=\"%s\"\n"
                           type mml-boundary))
+          (insert "\n")
           (setq cont (cddr cont))
           (while cont
             (insert "\n--" mml-boundary "\n")
@@ -654,14 +654,7 @@ called for this message.")
 	(when (string-match "[\"\\~/* \t\n]" value)
 	  (setq value (prin1-to-string value)))
 	(insert (format " %s=%s" key value)))))
-  (insert ">\n"))
-
-(defun mml-insert-empty-tag (name &rest plist)
-  "Insert an empty MML tag described by NAME and PLIST."
-  (when (symbolp name)
-    (setq name (symbol-name name)))
-  (apply #'mml-insert-tag name plist)
-  (insert "<#/" name ">\n"))
+  (insert ">\n<#/" name ">\n"))
 
 ;;; Attachment functions.
 
@@ -678,8 +671,8 @@ description of the attachment."
 	  (type (mml-minibuffer-read-type file))
 	  (description (mml-minibuffer-read-description)))
      (list file type description)))
-  (mml-insert-empty-tag 'part 'type type 'filename file
-			'disposition "attachment" 'description description))
+  (mml-insert-tag 'part 'type type 'filename file 'disposition "attachment"
+		  'description description))
 
 (defun mml-attach-buffer (buffer &optional type description)
   "Attach a buffer to the outgoing MIME message.
@@ -689,8 +682,8 @@ See `mml-attach-file' for details of operation."
 	  (type (mml-minibuffer-read-type buffer "text/plain"))
 	  (description (mml-minibuffer-read-description)))
      (list buffer type description)))
-  (mml-insert-empty-tag 'part 'type type 'buffer buffer
-			'disposition "attachment" 'description description))
+  (mml-insert-tag 'part 'type type 'buffer buffer 'disposition "attachment"
+		  'description description))
 
 (defun mml-attach-external (file &optional type description)
   "Attach an external file into the buffer.
@@ -701,8 +694,8 @@ TYPE is the MIME type to use."
 	  (type (mml-minibuffer-read-type file))
 	  (description (mml-minibuffer-read-description)))
      (list file type description)))
-  (mml-insert-empty-tag 'external 'type type 'name file
-			'disposition "attachment" 'description description))
+  (mml-insert-tag 'external 'type type 'name file 'disposition "attachment"
+		  'description description))
 
 (defun mml-insert-multipart (&optional type)
   (interactive (list (completing-read "Multipart type (default mixed): "
@@ -711,7 +704,7 @@ TYPE is the MIME type to use."
 		     nil nil "mixed")))
   (or type
       (setq type "mixed"))
-  (mml-insert-empty-tag "multipart" 'type type)
+  (mml-insert-tag "multipart" 'type type)
   (forward-line -1))
 
 (defun mml-preview (&optional raw)
